@@ -14,7 +14,7 @@ from typing import Any, Callable, List, Optional, Tuple, Type, Union
 from .namespace import is_empty_namespace, Namespace, split_key, split_key_leaf, split_key_root
 from .optionals import FilesCompleterMethod, get_config_read_mode
 from .type_checking import ArgumentParser, _ArgumentGroup
-from .typing import path_type
+from .typing import get_import_path, path_type
 from .util import (
     DirectedGraph,
     yamlParserError,
@@ -25,7 +25,7 @@ from .util import (
     NoneType,
     indent_text,
     Path,
-    _load_config,
+    _parse_value_or_config,
     _issubclass,
 )
 
@@ -297,12 +297,11 @@ class _ActionConfigLoad(Action):
 
     def _load_config(self, value, parser):
         try:
-            cfg, cfg_path = _load_config(value)
+            cfg, cfg_path = _parse_value_or_config(value)
             if not isinstance(cfg, dict):
                 raise TypeError(f'Parser key "{self.dest}": Unable to load config "{value}"')
             with change_to_path_dir(cfg_path):
-                cfg = parser._config_from_dict(cfg, parent_key=self.dest)
-                parser._apply_actions(cfg, parent_key=self.dest)
+                cfg = parser._apply_actions(cfg, parent_key=self.dest)
             return cfg
         except (TypeError, yamlParserError, yamlScannerError) as ex:
             str_ex = indent_text(str(ex))
@@ -345,7 +344,7 @@ class _ActionHelpClass(Action):
 
     def print_help(self, call_args, val_class, dest):
         tmp = import_object('jsonargparse.ArgumentParser')()
-        tmp.add_class_arguments(val_class, dest)
+        tmp.add_class_arguments(val_class, dest, **self.sub_add_kwargs)
         _remove_actions(tmp, (_HelpAction, _ActionHelpClass, _ActionPrintConfig))
         tmp.print_help()
         call_args[0].exit()
@@ -355,9 +354,9 @@ class _ActionHelpClassPath(_ActionHelpClass):
 
     def update_init_kwargs(self, kwargs):
         if getattr(self._baseclass, '__origin__', None) == Union:
-            self._basename = '{'+', '.join(c.__name__ for c in self._baseclass.__args__)+'}'
+            self._basename = '{'+', '.join(get_import_path(c) for c in self._baseclass.__args__)+'}'
         else:
-            self._basename = self._baseclass.__name__
+            self._basename = get_import_path(self._baseclass)
         kwargs.update({
             'metavar': 'CLASS',
             'default': SUPPRESS,
@@ -538,6 +537,8 @@ class _ActionLink(Action):
             else:
                 attr = split_key_leaf(action.source[0][0])[1]
                 value = getattr(source_object, attr)
+                if action.compute_fn is not None:
+                    value = action.compute_fn(value)
             _ActionLink.set_target_value(action, value, cfg)
 
     @staticmethod
