@@ -9,7 +9,7 @@ from typing import Any, Callable, List, Optional, Set, Tuple, Type, Union
 from .actions import _ActionConfigLoad, _ActionHelpClass, _ActionHelpClassPath
 from .namespace import Namespace
 from .typehints import ActionTypeHint, ClassType, is_optional
-from .typing import is_final_class
+from .typing import get_import_path, is_final_class
 from .util import _issubclass
 from .optionals import (
     dataclasses_support,
@@ -346,8 +346,9 @@ class SignatureArguments:
                     if ActionTypeHint.is_subclass_typehint(annotation):
                         help_action = _ActionHelpClassPath
                         prefix = name + '.init_args.'
-                    group.add_argument(f'--{dest}.help', action=help_action(baseclass=annotation))
+                    help_action = group.add_argument(f'--{dest}.help', action=help_action(baseclass=annotation))
                     subclass_skip = {s[len(prefix):] for s in skip if s.startswith(prefix)}
+                    help_action.sub_add_kwargs = {'fail_untyped': fail_untyped, 'skip': subclass_skip}
                 action = group.add_argument(opt_str, **kwargs)
                 action.sub_add_kwargs = {
                     'fail_untyped': fail_untyped,
@@ -481,9 +482,10 @@ class SignatureArguments:
         else:
             skip = set(nested_key+'.init_args.'+s for s in skip)
         param = Namespace(name=nested_key, _kind=None, annotation=Union[baseclass])
+        str_baseclass = '{' + ', '.join(get_import_path(x) for x in baseclass) + '}'
         kwargs.update({
             'metavar': metavar,
-            'help': (help % {'baseclass_name': str(baseclass)}),
+            'help': (help % {'baseclass_name': str_baseclass}),
         })
         if 'default' not in kwargs:
             kwargs['default'] = SUPPRESS
