@@ -53,8 +53,9 @@ from .util import (
     yamlScannerError,
     indent_text,
     _issubclass,
-    _load_config,
+    _parse_value_or_config,
     warning,
+    lenient_check,
 )
 
 
@@ -65,6 +66,9 @@ Literal = getattr(typing, 'Literal', False)
 
 
 root_types = {
+    str,
+    int,
+    float,
     bool,
     Any,
     Literal,
@@ -94,6 +98,7 @@ mapping_origin_types = {Dict, dict, Mapping, MutableMapping, abcMapping, abcMuta
 
 
 subclass_arg_parser: ContextVar = ContextVar('subclass_arg_parser')
+sub_defaults: ContextVar = ContextVar('sub_defaults', default=False)
 
 
 class ActionTypeHint(Action):
@@ -163,31 +168,11 @@ class ActionTypeHint(Action):
         return inspect.isclass(typehint) and typehint not in not_subclass_types and typehint_origin is None
 
 
-    @staticmethod
-    def is_final_class_typehint(typehint):
-        typehint = typehint_from_action(typehint)
-        typehint_origin = getattr(typehint, '__origin__', None)
-        if typehint_origin == Union:
-            subtypes = [a for a in typehint.__args__ if a != NoneType]
-            if len(subtypes) == 1:
-                typehint = subtypes[0]
-        return is_final_class(typehint)
-
-
     @staticmethod
     def is_subclass_typehint(typehint):
         return ActionTypeHint.is_class_typehint(typehint, True)
 
 
-    @staticmethod
-    def is_mapping_class_typehint(typehint):
-        typehint = typehint_from_action(typehint)
-        typehint_origin = getattr(typehint, '__origin__', None)
-        if typehint_origin not in mapping_origin_types:
-            return False
-        return ActionTypeHint.is_class_typehint(getattr(typehint, '__args__')[1])
-
-
     @staticmethod
     def is_mapping_typehint(typehint):
         typehint_origin = getattr(typehint, '__origin__', typehint)
@@ -239,6 +224,22 @@ class ActionTypeHint(Action):
         yield
 
 
+    @staticmethod
+    @contextmanager
+    def sub_defaults_context():
+        t = sub_defaults.set(True)
+        try:
+            yield
+        finally:
+            sub_defaults.reset(t)
+
+
+    @staticmethod
+    def add_sub_defaults(parser, cfg):
+        with ActionTypeHint.sub_defaults_context():
+            parser._apply_actions(cfg)
+
+
     def serialize(self, value):
         sub_add_kwargs = getattr(self, 'sub_add_kwargs', {})
         if _is_action_value_list(self):
@@ -302,16 +303,20 @@ class ActionTypeHint(Action):
             try:
                 orig_val = val
                 try:
-                    val, config_path = _load_config(val, enable_path=self._enable_path)
+                    val, config_path = _parse_value_or_config(val, enable_path=self._enable_path)
                 except (yamlParserError, yamlScannerError):
                     config_path = None
-                path_meta = val.pop('__path__') if isinstance(val, dict) and '__path__' in val else None
+                path_meta = val.pop('__path__', None) if isinstance(val, dict) else None
                 sub_add_kwargs = getattr(self, 'sub_add_kwargs', {})
                 try:
                     with change_to_path_dir(config_path):
                         val = adapt_typehints(val, self._typehint, sub_add_kwargs=sub_add_kwargs)
                 except ValueError as ex:
-                    if isinstance(val, (int, float)) and config_path is None:
+                    val_is_int_float_or_none = isinstance(val, (int, float)) or val is None
+                    if lenient_check.get():
+                        value[num] = orig_val if val_is_int_float_or_none else val
+                        continue
+                    if val_is_int_float_or_none and config_path is None:
                         val = adapt_typehints(orig_val, self._typehint, sub_add_kwargs=sub_add_kwargs)
                     else:
                         if self._enable_path and config_path is None and isinstance(orig_val, str):
@@ -329,21 +334,14 @@ class ActionTypeHint(Action):
         return value if islist else value[0]
 
 
-    def instantiate_classes(self, val):
+    def instantiate_classes(self, value):
+        islist = _is_action_value_list(self)
+        if not islist:
+            value = [value]
         sub_add_kwargs = getattr(self, 'sub_add_kwargs', {})
-        return adapt_typehints(val, self._typehint, instantiate_classes=True, sub_add_kwargs=sub_add_kwargs)
-
-
-    def get_class_and_init_args(self, value):
-        final_class = ActionTypeHint.is_final_class_typehint(self)
-        if final_class:
-            typehint_origin = getattr(self._typehint, '__origin__', None)
-            class_type = [a for a in self._typehint.__args__ if a != NoneType][0] if typehint_origin == Union else self._typehint
-            init_args = value
-        else:
-            class_type = import_object(value['class_path'])
-            init_args = value.get('init_args')
-        return class_type, init_args, final_class
+        for num, val in enumerate(value):
+            value[num] = adapt_typehints(val, self._typehint, instantiate_classes=True, sub_add_kwargs=sub_add_kwargs)
+        return value if islist else value[0]
 
 
     @staticmethod
@@ -399,7 +397,7 @@ def adapt_typehints(val, typehint, serialize=False, instantiate_classes=False, s
             val = adapt_typehints(val, type_val, **adapt_kwargs)
         elif isinstance(val, str):
             try:
-                val, _ = _load_config(val, enable_path=False)
+                val, _ = _parse_value_or_config(val, enable_path=False)
             except (yamlParserError, yamlScannerError):
                 pass
 
@@ -411,7 +409,10 @@ def adapt_typehints(val, typehint, serialize=False, instantiate_classes=False, s
     # Basic types
     elif typehint in leaf_types:
         if not isinstance(val, typehint):
-            raise ValueError(f'Expected a {typehint} but got "{val}"')
+            if typehint is float and isinstance(val, int):
+                val = float(val)
+            else:
+                raise ValueError(f'Expected a {typehint} but got "{val}"')
 
     # Registered types
     elif typehint in registered_types:
@@ -554,10 +555,16 @@ def adapt_typehints(val, typehint, serialize=False, instantiate_classes=False, s
 
 
 def is_class_object(val):
-    return isinstance(val, (dict, Namespace)) and 'class_path' in val
+    is_class = isinstance(val, (dict, Namespace)) and 'class_path'
+    if is_class:
+        keys = getattr(val, '__dict__', val).keys()
+        is_class = len(set(keys)-{'class_path', 'init_args', '__path__'}) == 0
+    return is_class
 
 
 def adapt_class_type(val_class, init_args, serialize, instantiate_classes, sub_add_kwargs):
+    if not isinstance(init_args, Namespace):
+        raise ValueError(f'Unexpected init_args value: "{init_args}".')
     parser = ActionTypeHint.get_class_parser(val_class, sub_add_kwargs)
 
     # No need to re-create the linked arg but just "inform" the corresponding parser actions that it exists upstream.
@@ -584,7 +591,7 @@ def adapt_class_type(val_class, init_args, serialize, instantiate_classes, sub_a
     if serialize:
         init_args = None if is_empty_namespace(init_args) else yaml.safe_load(parser.dump(init_args))
     else:
-        init_args = parser.parse_object(init_args)
+        init_args = parser.parse_object(init_args, defaults=sub_defaults.get())
     return init_args
 
 
@@ -630,9 +637,21 @@ def typehint_metavar(typehint):
     return metavar
 
 
+def check_lazy_kwargs(class_type: Type, lazy_kwargs: dict):
+    if lazy_kwargs:
+        from .core import ArgumentParser
+        parser = ArgumentParser(error_handler=None)
+        parser.add_class_arguments(class_type)
+        try:
+            parser.parse_object(lazy_kwargs)
+        except ParserError as ex:
+            raise ValueError(str(ex)) from ex
+
+
 class LazyInitBaseClass:
 
     def __init__(self, class_type: Type, lazy_kwargs: dict):
+        check_lazy_kwargs(class_type, lazy_kwargs)
         self._lazy_class_type = class_type
         self._lazy_class_path = get_import_path(class_type)
         self._lazy_kwargs = lazy_kwargs
