@@ -1,22 +1,22 @@
 """Collection of general functions and classes."""
 
-import os
-import re
-import sys
-import stat
-import yaml
 import inspect
 import logging
+import os
+import re
+import stat
+import sys
+import warnings
+import yaml
+from argparse import Action
 from collections import defaultdict
-from copy import deepcopy
-from typing import Dict, Any, Optional, Union
 from contextlib import contextmanager, redirect_stderr
-from argparse import Namespace
+from contextvars import ContextVar
+from typing import Any, Optional, Tuple, Union
 from yaml.parser import ParserError as yamlParserError
 from yaml.scanner import ScannerError as yamlScannerError
 
 from .optionals import (
-    ModuleNotFound,
     url_support,
     import_requests,
     import_url_validator,
@@ -24,15 +24,12 @@ from .optionals import (
     import_fsspec,
     get_config_read_mode,
 )
+from .type_checking import ArgumentParser
 
 
 __all__ = [
     'ParserError',
     'null_logger',
-    'dict_to_namespace',
-    'namespace_to_dict',
-    'meta_keys',
-    'strip_meta',
     'usage_and_exit_error_handler',
     'Path',
     'LoggerProperty',
@@ -42,9 +39,6 @@ __all__ = [
 null_logger = logging.Logger('jsonargparse_null_logger')
 null_logger.addHandler(logging.NullHandler())
 
-meta_keys = {'__default_config__', '__path__', '__orig__'}
-
-empty_namespace = Namespace()
 
 NoneType = type(None)
 
@@ -54,8 +48,20 @@ class ParserError(Exception):
     pass
 
 
-def _load_config(value, enable_path=True, flat_namespace=True):
-    """Parses yaml config in a string or a path"""
+class JsonargparseWarning(UserWarning):
+    pass
+
+
+def warning(message, category=JsonargparseWarning, stacklevel=1):
+    warnings.warn(
+        re.sub('\n\n+', '\n\n', re.sub('\n +', '\n  ', message)),
+        category=category,
+        stacklevel=stacklevel+1,
+    )
+
+
+def _parse_value_or_config(value: Any, enable_path: bool = True) -> Tuple[Any, Optional['Path']]:
+    """Parses yaml/json config in a string or a path"""
     cfg_path = None
     if isinstance(value, str) and value.strip() != '':
         parsed_val = yaml.safe_load(value)
@@ -68,213 +74,25 @@ def _load_config(value, enable_path=True, flat_namespace=True):
             pass
         else:
             value = yaml.safe_load(cfg_path.get_content())
-
-    if flat_namespace and isinstance(value, dict):
-        value = _dict_to_flat_namespace(value)
-        if cfg_path is not None:
-            setattr(value, '__path__', cfg_path)
-        return value
-
+    if isinstance(value, dict) and cfg_path is not None:
+        value['__path__'] = cfg_path
     return value, cfg_path
 
 
-def get_key_value_from_flat_dict(cfg, key):
-    value = cfg.get(key)
-    if value is not None:
-        return value
-    value = {k[len(key)+1:]: v for k, v in cfg.items() if k.startswith(key+'.')}
-    return _flat_namespace_to_dict(Namespace(**value))
-
-
-def update_key_value_in_flat_dict(cfg, key, value):
-    if isinstance(value, dict):
-        value = vars(_dict_to_flat_namespace(value))
-        if key in cfg:
-            del cfg[key]
-        cfg.update({key+'.'+k: v for k, v in value.items()})
-    else:
-        cfg[key] = value
-
-
-def _get_key_value(cfg, key, parent=False):
-    """Gets the value for a given key in a config object (dict or argparse.Namespace)."""
-    def key_in_cfg(cfg, key):
-        if (isinstance(cfg, Namespace) and hasattr(cfg, key)) or \
-           (isinstance(cfg, dict) and key in cfg):
-            return True
-        return False
-
-    c = cfg
-    k = key
-    while '.' in k and not key_in_cfg(c, k):
-        kp, k = k.split('.', 1)
-        c = c[kp] if isinstance(c, dict) else getattr(c, kp)
-
-    v = c[k] if isinstance(c, dict) else getattr(c, k)
-    return (v, c, k) if parent else v
-
-
-def _flat_namespace_to_dict(cfg_ns:Namespace) -> Dict[str, Any]:
-    """Converts a flat namespace into a nested dictionary.
-
-    Args:
-        cfg_ns: The configuration to process.
-
-    Returns:
-        The nested configuration dictionary.
-    """
-    def raise_conflicting_base(base):
-        raise ParserError('Conflicting namespace base: '+base)
-
-    nested_keys = {k.rsplit('.', 1)[0]+'.' for k in vars(cfg_ns).keys() if '.' in k}
-    skip_keys = {k for k, v in vars(cfg_ns).items() if v is None and k+'.' in nested_keys}
-
-    cfg_ns = deepcopy(cfg_ns)
-    cfg_dict = {}
-    for k, v in vars(cfg_ns).items():
-        if k in skip_keys:
-            continue
-        ksplit = k.split('.')
-        if len(ksplit) == 1:
-            if k in cfg_dict:
-                raise_conflicting_base(k)
-            #elif isinstance(v, list) and any([isinstance(x, Namespace) for x in v]):
-            #    cfg_dict[k] = [namespace_to_dict(x) for x in v]
-            #elif isinstance(v, Namespace):
-            #    cfg_dict[k] = vars(v)  # type: ignore
-            elif not (v is None and k in cfg_dict):
-                cfg_dict[k] = v
-        else:
-            kdict = cfg_dict
-            for num, kk in enumerate(ksplit[:len(ksplit)-1]):
-                if kk not in kdict or kdict[kk] is None or kdict[kk] == empty_namespace:
-                    kdict[kk] = {}
-                elif not isinstance(kdict[kk], dict):
-                    raise_conflicting_base('.'.join(ksplit[:num+1])+', expected dict but is '+str(kdict[kk]))
-                kdict = kdict[kk]
-            if ksplit[-1] in kdict and kdict[ksplit[-1]] is not None and v != Namespace():
-                raise_conflicting_base(k)
-            #if isinstance(v, list) and any([isinstance(x, Namespace) for x in v]):
-            #    kdict[ksplit[-1]] = [namespace_to_dict(x) for x in v]
-            #elif not (v is None and ksplit[-1] in kdict):
-            if not ((v is None or v == Namespace()) and ksplit[-1] in kdict):
-                kdict[ksplit[-1]] = v
-    return cfg_dict
-
-
-def _dict_to_flat_namespace(cfg_dict:Dict[str, Any]) -> Namespace:
-    """Converts a nested dictionary into a flat namespace.
-
-    Args:
-        cfg_dict: The configuration to process.
-
-    Returns:
-        The configuration namespace.
-    """
-    cfg_dict = deepcopy(cfg_dict)
-    cfg_ns = {}
-
-    def flatten_dict(cfg, base=None):
-        for key, val in cfg.items():
-            kbase = key if base is None else base+'.'+key
-            if isinstance(val, dict) and val != {} and all(isinstance(k, str) for k in val.keys()):
-                flatten_dict(val, kbase)
-            else:
-                cfg_ns[kbase] = val
-
-    flatten_dict(cfg_dict)
-
-    return Namespace(**cfg_ns)
-
-
-def dict_to_namespace(cfg_dict:Dict[str, Any]) -> Namespace:
-    """Converts a nested dictionary into a nested namespace.
-
-    Args:
-        cfg_dict: The configuration to process.
-
-    Returns:
-        The nested configuration namespace.
-    """
-    cfg_dict = deepcopy(cfg_dict)
-    def expand_dict(cfg):
-        for k, v in cfg.items():
-            if isinstance(v, dict) and all(isinstance(k, str) for k in v.keys()):
-                cfg[k] = expand_dict(v)
-            elif isinstance(v, list):
-                for nn, vv in enumerate(v):
-                    if isinstance(vv, dict) and all(isinstance(k, str) for k in vv.keys()):
-                        cfg[k][nn] = expand_dict(vv)
-        return Namespace(**cfg)
-    return expand_dict(cfg_dict)
-
-
-def namespace_to_dict(cfg_ns:Namespace) -> Dict[str, Any]:
-    """Converts a nested namespace into a nested dictionary.
-
-    Args:
-        cfg_ns: The configuration to process.
-
-    Returns:
-        The nested configuration dictionary.
-    """
-    cfg_ns = deepcopy(cfg_ns)
-    def expand_namespace(cfg):
-        if not isinstance(cfg, dict):
-            cfg = dict(vars(cfg))
-        for k, v in cfg.items():
-            if isinstance(v, Namespace):
-                cfg[k] = expand_namespace(v)
-            elif isinstance(v, list):
-                for nn, vv in enumerate(v):
-                    if isinstance(vv, Namespace):
-                        cfg[k][nn] = expand_namespace(vv)
-        return cfg
-    return expand_namespace(cfg_ns)
-
-
-def strip_meta(cfg:Union[Namespace, Dict[str, Any]]) -> Dict[str, Any]:
-    """Removes all metadata keys from a configuration object.
-
-    Args:
-        cfg: The configuration object to strip.
-
-    Returns:
-        A copy of the configuration object without any metadata keys.
-    """
-    cfg = deepcopy(cfg)
-    if not isinstance(cfg, dict):
-        cfg = namespace_to_dict(cfg)
-
-    def strip_keys(cfg, base=None):
-        del_keys = []
-        for key, val in cfg.items():
-            kbase = str(key) if base is None else base+'.'+str(key)
-            if isinstance(val, dict):
-                strip_keys(val, kbase)
-            elif key in meta_keys:
-                del_keys.append(key)
-        for key in del_keys:
-            del cfg[key]
-
-    strip_keys(cfg)
-    return cfg
-
-
-def usage_and_exit_error_handler(self, message:str):
+def usage_and_exit_error_handler(parser: 'ArgumentParser', message: str) -> None:
     """Error handler to get the same behavior as in argparse.
 
     Args:
-        self (ArgumentParser): The ArgumentParser object.
+        parser: The parser object.
         message: The message describing the error being handled.
     """
-    self.print_usage(sys.stderr)
-    args = {'prog': self.prog, 'message': message}
+    parser.print_usage(sys.stderr)
+    args = {'prog': parser.prog, 'message': message}
     sys.stderr.write('%(prog)s: error: %(message)s\n' % args)
-    self.exit(2)
+    parser.exit(2)
 
 
-def _get_env_var(parser, action) -> str:
+def _get_env_var(parser: 'ArgumentParser', action: 'Action') -> str:
     """Returns the environment variable for a given parser and action."""
     if hasattr(parser, '_parser'):
         parser = parser._parser
@@ -288,15 +106,32 @@ def _issubclass(cls, class_or_tuple):
     return inspect.isclass(cls) and issubclass(cls, class_or_tuple)
 
 
-def import_object(name):
+def _check_valid_dump_format(value: str):
+    if value not in {'parser_mode', 'yaml', 'json_indented', 'json'}:
+        raise ValueError(f'Unknown output format "{value}".')
+
+
+def import_object(name: str):
     """Returns an object in a module given its dot import path."""
-    if not isinstance(name, str):
+    if not isinstance(name, str) or '.' not in name:
         raise ValueError('Expected a dot import path string')
     name_module, name_object = name.rsplit('.', 1)
     module = __import__(name_module, fromlist=[name_object])
     return getattr(module, name_object)
 
 
+lenient_check: ContextVar = ContextVar('lenient_check', default=False)
+
+
+@contextmanager
+def _lenient_check_context(caller=None):
+    t = lenient_check.set(False if caller == 'argcomplete' else True)
+    try:
+        yield
+    finally:
+        lenient_check.reset(t)
+
+
 @contextmanager
 def _suppress_stderr():
     """A context manager that redirects stderr to devnull."""
@@ -306,7 +141,7 @@ def _suppress_stderr():
 
 
 @contextmanager
-def change_to_path_dir(path):
+def change_to_path_dir(path: Optional['Path']):
     """A context manager for running code in the directory of a path."""
     chdir = path is not None and not (path.is_url or path.is_fsspec)
     if chdir:
@@ -319,11 +154,11 @@ def change_to_path_dir(path):
             os.chdir(cwd)
 
 
-def indent_text(text):
+def indent_text(text: str) -> str:
     return text.replace('\n', '\n  ')
 
 
-def known_to_fsspec(path):
+def known_to_fsspec(path: str) -> bool:
     import_fsspec('known_to_fsspec')
     from fsspec.registry import known_implementations
     for protocol in known_implementations.keys():
@@ -356,7 +191,7 @@ class DirectedGraph:
         exploring[source] = True
         for target in self.edges_dict[source]:
             if exploring[target]:
-                raise ValueError('Graph has cycles, found while checking '+self.nodes[source]+' --> '+self.nodes[target])
+                raise ValueError(f'Graph has cycles, found while checking {self.nodes[source]} --> '+self.nodes[target])
             elif not visited[target]:
                 self.topological_sort(target, exploring, visited, order)
         visited[source] = True
@@ -425,7 +260,7 @@ class Path:
                 try:
                     requests.head(abs_path).raise_for_status()
                 except requests.HTTPError as ex:
-                    raise TypeError(abs_path+' HEAD not accessible :: '+str(ex)) from ex
+                    raise TypeError(f'{abs_path} HEAD not accessible :: {ex}') from ex
         elif not skip_check and is_fsspec:
             fsspec_mode = ''.join(c for c in mode if c in {'r','w'})
             if fsspec_mode:
@@ -478,8 +313,8 @@ class Path:
         self.abs_path = abs_path
         self.cwd = cwd
         self.mode = mode
-        self.is_url = is_url  # type: bool
-        self.is_fsspec = is_fsspec  # type: bool
+        self.is_url: bool = is_url
+        self.is_fsspec: bool = is_fsspec
         self.skip_check = skip_check
 
     def __str__(self):
@@ -563,13 +398,13 @@ class LoggerProperty:
             level = logging.WARNING
             if isinstance(logger, dict) and 'level' in logger:
                 if logger['level'] not in levels:
-                    raise ValueError('Logger level must be one of '+str(levels)+'.')
+                    raise ValueError(f'Logger level must be one of {levels}.')
                 level = getattr(logging, logger['level'])
             if isinstance(logger, bool) or (isinstance(logger, dict) and 'name' not in logger):
                 try:
                     import reconplogger
                     logger = reconplogger.logger_setup(level=level)
-                except (ImportError, ModuleNotFound, ValueError):
+                except (ImportError, ValueError):
                     pass
             if not isinstance(logger, logging.Logger):
                 name = type(self).__name__
