@@ -10,6 +10,9 @@ testing_package = os.path.basename(os.path.dirname(os.path.realpath(__file__)))
 
 
 def run_tests():
+    filter_action = 'default'
+    warnings.simplefilter(filter_action)
+    os.environ['PYTHONWARNINGS'] = filter_action
     tests = unittest.defaultTestLoader.discover(testing_package, pattern='*_tests.py')
     if not unittest.TextTestRunner(verbosity=2).run(tests).wasSuccessful():
         sys.exit(True)
@@ -21,8 +24,6 @@ def run_test_coverage():
     except:
         print('error: coverage package not found, run_test_coverage requires it.')
         sys.exit(True)
-    warnings.simplefilter('default')
-    os.environ['PYTHONWARNINGS'] = 'default'
     package_source = os.path.dirname(__file__.replace('_tests', ''))
     cov = coverage.Coverage(source=[package_source])
     cov.start()
