@@ -2,6 +2,7 @@ import os
 import shutil
 import tempfile
 import unittest
+from contextlib import contextmanager
 from unittest import mock
 from jsonargparse import *
 from jsonargparse.typing import *
@@ -15,14 +16,13 @@ from jsonargparse.optionals import (
     fsspec_support, import_fsspec,
     get_config_read_mode,
     ruyaml_support, import_ruyaml,
-    ModuleNotFound,
 )
 
 
 try:
     import responses
     responses_activate = responses.activate
-except (ImportError, ModuleNotFound):
+except ImportError:
     def nothing_decorator(func):
         return func
     responses = False  # type: ignore
@@ -32,6 +32,16 @@ except (ImportError, ModuleNotFound):
 os.environ['COLUMNS'] = '150'
 
 
+@contextmanager
+def mock_module(**kwargs):
+    __module__ = 'jsonargparse_tests'
+    for component in kwargs.values():
+        component.__module__ = __module__
+    import jsonargparse_tests
+    with mock.patch.multiple(jsonargparse_tests, create=True, **kwargs):
+        yield __module__
+
+
 class TempDirTestCase(unittest.TestCase):
 
     def setUp(self):
@@ -42,7 +52,10 @@ class TempDirTestCase(unittest.TestCase):
 
     def tearDown(self):
         os.chdir(self.cwd)
-        shutil.rmtree(self.tmpdir)
+        try:
+            shutil.rmtree(self.tmpdir)
+        except PermissionError:
+            pass
 
 
 def example_parser():
