@@ -1,9 +1,9 @@
 #!/usr/bin/env python3
 
+import calendar
 from enum import Enum
 from io import StringIO
 from jsonargparse_tests.base import *
-from jsonargparse import ArgumentParser
 
 
 class DeprecatedTests(unittest.TestCase):
@@ -111,8 +111,32 @@ class DeprecatedTests(unittest.TestCase):
         self.assertEqual('fr', get_config_read_mode())
 
 
+    def test_instantiate_subclasses(self):
+        parser = ArgumentParser(error_handler=None)
+        parser.add_argument('--cal', type=calendar.Calendar)
+        cfg = parser.parse_object({'cal':{'class_path': 'calendar.Calendar'}})
+        cfg_init = parser.instantiate_subclasses(cfg)
+        self.assertIsInstance(cfg_init['cal'], calendar.Calendar)
+
+
 class DeprecatedTempDirTests(TempDirTestCase):
 
+    def test_parse_as_dict(self):
+        with open('config.json', 'w') as f:
+            f.write('{}')
+        parser = ArgumentParser(parse_as_dict=True, default_meta=False)
+        self.assertEqual({}, parser.parse_args([]))
+        self.assertEqual({}, parser.parse_env([]))
+        self.assertEqual({}, parser.parse_string('{}'))
+        self.assertEqual({}, parser.parse_object({}))
+        self.assertEqual({}, parser.parse_path('config.json'))
+        self.assertEqual({}, parser.instantiate_classes({}))
+        self.assertEqual('{}\n', parser.dump({}))
+        parser.save({}, 'config.yaml')
+        with open('config.yaml') as f:
+            self.assertEqual('{}\n', f.read())
+
+
     def test_ActionPath(self):
         os.mkdir(os.path.join(self.tmpdir, 'example'))
         rel_yaml_file = os.path.join('..', 'example', 'example.yaml')
