@@ -334,6 +334,30 @@ class SignaturesTests(unittest.TestCase):
         self.assertIn('cal: <calendar.Calendar object at ', dump)
 
 
+    def test_subclass_help(self):
+        class MyCal(calendar.Calendar):
+            def __init__(self, *args, param, **kwargs):
+                self.param = param
+                super().__init__(*args, **kwargs)
+
+        from jsonargparse_tests import signatures_tests
+        setattr(signatures_tests, 'MyCal', MyCal)
+        args = ['--cal.help=jsonargparse_tests.signatures_tests.MyCal']
+
+        parser = ArgumentParser()
+        parser.add_subclass_arguments(calendar.Calendar, 'cal', skip={'param'})
+
+        out = StringIO()
+        with redirect_stdout(out), self.assertRaises(SystemExit):
+            parser.parse_args(args)
+        self.assertIn('--cal.init_args.firstweekday', out.getvalue())
+        self.assertNotIn('param', out.getvalue())
+
+        parser = ArgumentParser(error_handler=None)
+        parser.add_subclass_arguments(calendar.Calendar, 'cal')
+        self.assertRaises(ValueError, lambda: parser.parse_args(args))
+
+
     def test_add_subclass_discard_init_args(self):
         parser = ArgumentParser(error_handler=None)
         parser.add_subclass_arguments(calendar.Calendar, 'cal')
@@ -572,7 +596,7 @@ class SignaturesTests(unittest.TestCase):
         parser = ArgumentParser(error_handler=None)
         parser.add_class_arguments(ClassB, 'b')
 
-        self.assertEqual(-3.2, parser.get_defaults().b.b2.a2)
+        self.assertEqual(parser.get_defaults().b.b2, Namespace(a1=1, a2=-3.2))
         cfg = parser.parse_args(['--b.b2={"a2": 6.7}'])
         self.assertEqual(cfg.b.b2, Namespace(a1=1, a2=6.7))
         self.assertEqual(cfg, parser.parse_string(parser.dump(cfg)))
@@ -1026,9 +1050,11 @@ class SignaturesTests(unittest.TestCase):
 
         # Link object attribute
         parser = make_parser_1()
+        parser.link_arguments('c.c2', 'b.b3', compute_fn=lambda v: f'"{v}"', apply_on='instantiate')
         cfg = parser.parse_args([])
         cfg = parser.instantiate_classes(cfg)
-        self.assertEqual(cfg['a'].a1, 6)
+        self.assertEqual(cfg.a.a1, 6)
+        self.assertEqual(cfg.b.b3, '"8"')
 
         # Link all group arguments
         parser = make_parser_1()
@@ -1110,6 +1136,46 @@ class SignaturesTests(unittest.TestCase):
         self.assertEqual(cfg['a'].a2, 3)
 
 
+    def test_dict_type_nested_in_two_level_subclasses(self):
+
+        class Module:
+            pass
+
+        class Network(Module):
+            def __init__(self, sub_network: Module, some_dict: Dict[str, Any] = {}):
+                pass
+
+        class Model:
+            def __init__(self, encoder: Module):
+                pass
+
+        with mock_module(Module=Module, Network=Network, Model=Model) as module:
+
+            config = f"""model:
+              encoder:
+                class_path: {module}.Network
+                init_args:
+                  some_dict:
+                    a: 1
+                  sub_network:
+                    class_path: {module}.Network
+                    init_args:
+                      some_dict:
+                        b: 2
+                      sub_network:
+                        class_path: {module}.Module
+            """
+
+            parser = ArgumentParser(error_handler=None)
+            parser.add_argument('--config', action=ActionConfigFile)
+            parser.add_class_arguments(Model, 'model')
+
+            cfg = parser.parse_args([f'--config={config}'])
+            self.assertEqual(cfg.model.encoder.init_args.some_dict, {'a': 1})
+            self.assertEqual(cfg.model.encoder.init_args.sub_network.init_args.some_dict, {'b': 2})
+            self.assertEqual(cfg.model.as_dict(), yaml.safe_load(config)['model'])
+
+
 class SignaturesConfigTests(TempDirTestCase):
 
     def test_add_function_arguments_config(self):
@@ -1219,6 +1285,38 @@ class SignaturesConfigTests(TempDirTestCase):
             self.assertEqual({'class_path': 'calendar.Calendar', 'init_args': {'firstweekday': 0}}, cal)
 
 
+    def test_subclass_required_param_with_default_config_files(self):
+
+        class SubModule:
+            def __init__(self, p1: int, p2: int = 2, p3: int = 3):
+                pass
+
+        class Model:
+            def __init__(self, sub_module: SubModule):
+                pass
+
+        with mock_module(SubModule=SubModule, Model=Model) as module:
+
+            defaults = f"""model:
+              sub_module:
+                class_path: {module}.SubModule
+                init_args:
+                  p1: 4
+                  p2: 5
+            """
+            expected = yaml.safe_load(defaults.replace('p2: 5', 'p2: 7'))['model']
+            expected['sub_module']['init_args']['p3'] = 3
+
+            with open('defaults.yaml', 'w') as f:
+                f.write(defaults)
+
+            parser = ArgumentParser(error_handler=None, default_config_files=['defaults.yaml'])
+            parser.add_class_arguments(Model, 'model')
+
+            cfg = parser.parse_args(['--model.sub_module.init_args.p2=7'])
+            self.assertEqual(cfg.model.as_dict(), expected)
+
+
 @unittest.skipIf(not dataclasses_support, 'dataclasses package is required')
 class DataclassesTests(unittest.TestCase):
 
