@@ -316,6 +316,46 @@ class TypeHintsTests(unittest.TestCase):
         self.assertIsNone(cfg['op'])
 
 
+    def test_class_type_without_defaults(self):
+        class MyCal(Calendar):
+            def __init__(self, p1: int = 1, p2: str = '2'):
+                pass
+
+        parser = ArgumentParser(error_handler=None)
+        parser.add_argument('--op', type=MyCal)
+
+        import jsonargparse_tests
+        setattr(jsonargparse_tests, 'MyCal', MyCal)
+
+        cfg = parser.parse_args(['--op.class_path=jsonargparse_tests.MyCal', '--op.init_args.p1=3'], defaults=False)
+        self.assertEqual(cfg.op, Namespace(class_path='jsonargparse_tests.MyCal', init_args=Namespace(p1=3)))
+
+
+    def test_class_type_required_params(self):
+        class MyCal(Calendar):
+            def __init__(self, p1: int, p2: str):
+                pass
+
+        with mock_module(MyCal=MyCal) as module:
+            parser = ArgumentParser(error_handler=None)
+            parser.add_argument('--op', type=MyCal, default=lazy_instance(MyCal))
+
+            cfg = parser.get_defaults()
+            self.assertEqual(cfg.op.class_path, f'{module}.MyCal')
+            self.assertEqual(cfg.op.init_args, Namespace(p1=None, p2=None))
+
+
+    def test_invalid_init_args_in_yaml(self):
+        config = """cal:
+            class_path: calendar.Calendar
+            init_args:
+        """
+        parser = ArgumentParser(error_handler=None)
+        parser.add_argument('--config', action=ActionConfigFile)
+        parser.add_argument('--cal', type=Calendar)
+        self.assertRaises(ParserError, lambda: parser.parse_args([f'--config={config}']))
+
+
     def test_typehint_serialize_list(self):
         parser = ArgumentParser()
         action = parser.add_argument('--list', type=Union[PositiveInt, List[PositiveInt]])
@@ -367,6 +407,14 @@ class TypeHintsTests(unittest.TestCase):
         register_type(uuid.UUID)
 
 
+    def test_lazy_instance_invalid_kwargs(self):
+        class MyClass:
+            def __init__(self, param: int = 1):
+                pass
+
+        self.assertRaises(ValueError, lambda: lazy_instance(MyClass, param='bad'))
+
+
 class TypeHintsTmpdirTests(TempDirTestCase):
 
     def test_path(self):
@@ -475,6 +523,11 @@ class TypeHintsTmpdirTests(TempDirTestCase):
         self.assertIn('class_path: calendar.Calendar\n', dump)
         self.assertIn('firstweekday: 3\n', dump)
 
+        cfg = parser.parse_args([])
+        self.assertEqual(cfg.data.cal.as_dict(), config)
+        cfg = parser.parse_args(['--data.cal.class_path=calendar.Calendar'], defaults=False)
+        self.assertEqual(cfg.data.cal, Namespace(class_path='calendar.Calendar'))
+
 
     def test_class_path_override_with_default_config_files(self):
 
